#!/usr/bin/env python3
# -*- coding: utf-8 -*-
#
# ORCID Hub documentation build configuration file, created by
# sphinx-quickstart on Tue Nov 28 17:15:32 2017.
#
# This file is execfile()d with the current directory set to its
# containing dir.
#
# Note that not all possible configuration values are present in this
# autogenerated file.
#
# All configuration values have a default; values that are commented out
# serve to show the default.

# If extensions (or modules to document with autodoc) are in another directory,
# add these directories to sys.path here. If the directory is relative to the
# documentation root, use os.path.abspath to make it absolute, like shown here.
#
import os
import sys
import datetime
import time
import logging
import recommonmark.parser
from recommonmark.transform import AutoStructify

BUILD_DATE = datetime.datetime.utcfromtimestamp(int(os.environ.get("SOURCE_DATE_EPOCH", time.time())))

# sys.path.insert(0, os.path.abspath("."))
# sys.path.insert(0, os.path.abspath("../.."))
_dir = os.path.dirname(__file__)
sys.path.append(os.path.dirname(_dir))
sys.path.append(_dir)

# disable logging to Sentry:
raven_logger = logging.getLogger("raven.base.Client")
raven_logger.setLevel(logging.CRITICAL)

# -- General configuration ------------------------------------------------

# If your documentation needs a minimal Sphinx version, state it here.
#
# needs_sphinx = "1.0"

# Add any Sphinx extension module names here, as strings. They can be
# extensions coming with Sphinx (named "sphinx.ext.*") or your custom
# ones.
extensions = [
    "sphinx.ext.autodoc",
    "sphinx.ext.doctest",
    "sphinx.ext.intersphinx",
    "m2r",
]

# Add any paths that contain templates here, relative to this directory.
templates_path = ["_templates"]

# The suffix(es) of source filenames.
# You can specify multiple suffix as a list of string:
#
# source_parsers = {
#     ".md": recommonmark.parser.CommonMarkParser,
# }
# source_suffix = {
#     ".rst": "restructuredtext",
#     ".md": "markdown",
# }
source_suffix = [".rst", ".md"]
# source_suffix = ".rst"

# The master toctree document.
master_doc = "index"

# General information about the project.
project = "ORCID Hub"
copyright = "2017, Royal Society of New Zealand"
author = "Jason Gush, Radomirs Cirskis, Roshan Pawar"

# The version info for the project you're documenting, acts as replacement for
# |version| and |release|, also used in various other places throughout the
# built documents.
#

try:
    from orcid_hub._version import __version__
    # The short X.Y version.
    version = '.'.join(__version__.split('.')[:2])
    # The full version, including alpha/beta/rc tags.
    release = __version__
except ImportError:
    # The short X.Y version.
    version = "3.0"
    # The full version, including alpha/beta/rc tags.
    release = "3.0a5"
    version = release = "dev"

# The language for content autogenerated by Sphinx. Refer to documentation
# for a list of supported languages.
#
# This is also used if you do content translation via gettext catalogs.
# Usually you set "language" from the command line for these cases.
language = None

# List of patterns, relative to source directory, that match files and
# directories to ignore when looking for source files.
# This patterns also effect to html_static_path and html_extra_path
exclude_patterns = ["_build", "Thumbs.db", ".DS_Store"]

# The name of the Pygments (syntax highlighting) style to use.
pygments_style = "sphinx"

# If true, `todo` and `todoList` produce output, else they produce nothing.
#todo_include_todos = False
todo_include_todos = True


# -- Options for HTML output ----------------------------------------------

# The theme to use for HTML and HTML Help pages.  See the documentation for
# a list of builtin themes.
#
html_theme = 'alabaster'

# Theme options are theme-specific and customize the look and feel of a theme
# further.  For a list of options available for each theme, see the
# documentation.
#
# html_theme_options = {}

# Add any paths that contain custom static files (such as style sheets) here,
# relative to this directory. They are copied after the builtin static files,
# so a file named "default.css" will overwrite the builtin "default.css".
html_static_path = ['_static']

html_logo = "../orcid_hub/static/images/logo.png"
html_favicon = "../orcid_hub/static/images/favicon-32x32.png"

# Custom sidebar templates, must be a dictionary that maps document names
# to template names.
#
# This is required for the alabaster theme
# refs: http://alabaster.readthedocs.io/en/latest/installation.html#sidebars
html_sidebars = {
    '**': [
        'relations.html',  # needs 'show_related': True theme option to display
        'searchbox.html',
    ]
}


# -- Options for HTMLHelp output ------------------------------------------

# Output file base name for HTML help builder.
htmlhelp_basename = 'ORCIDHubdoc'


# -- Options for LaTeX output ---------------------------------------------

latex_elements = {
    # The paper size ('letterpaper' or 'a4paper').
    #
    # 'papersize': 'letterpaper',

    # The font size ('10pt', '11pt' or '12pt').
    #
    # 'pointsize': '10pt',

    # Additional stuff for the LaTeX preamble.
    #
    # 'preamble': '',

    # Latex figure (float) alignment
    #
    # 'figure_align': 'htbp',
}

# Grouping the document tree into LaTeX files. List of tuples
# (source start file, target name, title,
#  author, documentclass [howto, manual, or own class]).
latex_documents = [
    (master_doc, 'ORCIDHub.tex', 'ORCID Hub Documentation',
     'Jason Gush, Radomirs Cirskis, Roshan Pawar', 'manual'),
]


# -- Options for manual page output ---------------------------------------

# One entry per manual page. List of tuples
# (source start file, name, description, authors, manual section).
man_pages = [
    (master_doc, 'orcidhub', 'ORCID Hub Documentation',
     [author], 1)
]


# -- Options for Texinfo output -------------------------------------------

# Grouping the document tree into Texinfo files. List of tuples
# (source start file, target name, title, author,
#  dir menu entry, description, category)
texinfo_documents = [
    (master_doc, 'ORCIDHub', 'ORCID Hub Documentation',
     author, 'ORCIDHub', 'One line description of project.',
     'Miscellaneous'),
]

# this should be at the bottom of conf.py
# def setup(app):
#     app.add_config_value(
#         'recommonmark_config', {
#             'url_resolver': lambda url: github_doc_root + url,
#             'auto_toc_tree_section': 'Contents',
#         }, True)
#     app.add_transform(AutoStructify)




# -*- coding: utf-8 -*-
"""Failover DB connection."""

import logging

from peewee import DatabaseError, InterfaceError, PostgresqlDatabase
from psycopg2 import OperationalError


class PgDbWithFailover(PostgresqlDatabase):
    """Postgres DB connection with a failover server."""

    def __init__(self, *args, failover_host=None, **kwargs):
        """Set up the connction paramers with an additional failover host name."""
        self.failover_host = failover_host
        super().__init__(*args, **kwargs)

    def connect(self):
        """Attempt to connect or reconnect."""
        if not self._local.closed:
            with self._conn_lock:
                with self.exception_wrapper:
                    try:
                        self._close(self._local.conn)
                    except Exception:
                        pass
                    self._local.closed = True
        super().connect()

    def _connect(self, database, encoding=None, **kwargs):
        """Attempt to connect to the DB.

        In case it's not available, 'fail over' to the back-up stand-by DB and propagete
        it to the master DB.
        """
        try:
            return super()._connect(database, encoding=encoding, **kwargs)
        except OperationalError as ex:
            logging.info("Failing over to %s", self.failover_host)

            if "could not connect to server" in ex.args[0] or "could not translate host name" in ex.args[0]:
                kwargs["host"] = self.failover_host
                conn = super()._connect(database, encoding=encoding, **kwargs)
                with conn.cursor() as cr:
                    cr.execute("SELECT promote_standby();")
                self.connect_kwargs[
                    "host"], self.failover_host = self.failover_host, self.connect_kwargs["host"]
                return conn
            else:
                raise ex

    def execute_sql(self, sql, params=None, require_commit=True):
        """Attempt to exectute a SQL statement. If it fails try to fail over ..."""
        try:
            return super().execute_sql(sql, params=params, require_commit=require_commit)
        except (DatabaseError, InterfaceError):
            self.connect()
            return super().execute_sql(sql, params=params, require_commit=False)



# coding: utf-8

"""
    ORCID Member

    No description provided (generated by Swagger Codegen https://github.com/swagger-api/swagger-codegen)

    OpenAPI spec version: Latest
    
    Generated by: https://github.com/swagger-api/swagger-codegen.git
"""


from pprint import pformat
from six import iteritems
import re


class Education(object):
    """
    NOTE: This class is auto generated by the swagger code generator program.
    Do not edit the class manually.
    """
    def __init__(self, created_date=None, last_modified_date=None, source=None, put_code=None, path=None, department_name=None, role_title=None, start_date=None, end_date=None, organization=None, visibility=None):
        """
        Education - a model defined in Swagger

        :param dict swaggerTypes: The key is attribute name
                                  and the value is attribute type.
        :param dict attributeMap: The key is attribute name
                                  and the value is json key in definition.
        """
        self.swagger_types = {
            'created_date': 'CreatedDate',
            'last_modified_date': 'LastModifiedDate',
            'source': 'Source',
            'put_code': 'int',
            'path': 'str',
            'department_name': 'str',
            'role_title': 'str',
            'start_date': 'FuzzyDate',
            'end_date': 'FuzzyDate',
            'organization': 'Organization',
            'visibility': 'str'
        }

        self.attribute_map = {
            'created_date': 'created-date',
            'last_modified_date': 'last-modified-date',
            'source': 'source',
            'put_code': 'put-code',
            'path': 'path',
            'department_name': 'department-name',
            'role_title': 'role-title',
            'start_date': 'start-date',
            'end_date': 'end-date',
            'organization': 'organization',
            'visibility': 'visibility'
        }

        self._created_date = created_date
        self._last_modified_date = last_modified_date
        self._source = source
        self._put_code = put_code
        self._path = path
        self._department_name = department_name
        self._role_title = role_title
        self._start_date = start_date
        self._end_date = end_date
        self._organization = organization
        self._visibility = visibility

    @property
    def created_date(self):
        """
        Gets the created_date of this Education.

        :return: The created_date of this Education.
        :rtype: CreatedDate
        """
        return self._created_date

    @created_date.setter
    def created_date(self, created_date):
        """
        Sets the created_date of this Education.

        :param created_date: The created_date of this Education.
        :type: CreatedDate
        """

        self._created_date = created_date

    @property
    def last_modified_date(self):
        """
        Gets the last_modified_date of this Education.

        :return: The last_modified_date of this Education.
        :rtype: LastModifiedDate
        """
        return self._last_modified_date

    @last_modified_date.setter
    def last_modified_date(self, last_modified_date):
        """
        Sets the last_modified_date of this Education.

        :param last_modified_date: The last_modified_date of this Education.
        :type: LastModifiedDate
        """

        self._last_modified_date = last_modified_date

    @property
    def source(self):
        """
        Gets the source of this Education.

        :return: The source of this Education.
        :rtype: Source
        """
        return self._source

    @source.setter
    def source(self, source):
        """
        Sets the source of this Education.

        :param source: The source of this Education.
        :type: Source
        """

        self._source = source

    @property
    def put_code(self):
        """
        Gets the put_code of this Education.

        :return: The put_code of this Education.
        :rtype: int
        """
        return self._put_code

    @put_code.setter
    def put_code(self, put_code):
        """
        Sets the put_code of this Education.

        :param put_code: The put_code of this Education.
        :type: int
        """

        self._put_code = put_code

    @property
    def path(self):
        """
        Gets the path of this Education.

        :return: The path of this Education.
        :rtype: str
        """
        return self._path

    @path.setter
    def path(self, path):
        """
        Sets the path of this Education.

        :param path: The path of this Education.
        :type: str
        """

        self._path = path

    @property
    def department_name(self):
        """
        Gets the department_name of this Education.

        :return: The department_name of this Education.
        :rtype: str
        """
        return self._department_name

    @department_name.setter
    def department_name(self, department_name):
        """
        Sets the department_name of this Education.

        :param department_name: The department_name of this Education.
        :type: str
        """

        self._department_name = department_name

    @property
    def role_title(self):
        """
        Gets the role_title of this Education.

        :return: The role_title of this Education.
        :rtype: str
        """
        return self._role_title

    @role_title.setter
    def role_title(self, role_title):
        """
        Sets the role_title of this Education.

        :param role_title: The role_title of this Education.
        :type: str
        """

        self._role_title = role_title

    @property
    def start_date(self):
        """
        Gets the start_date of this Education.

        :return: The start_date of this Education.
        :rtype: FuzzyDate
        """
        return self._start_date

    @start_date.setter
    def start_date(self, start_date):
        """
        Sets the start_date of this Education.

        :param start_date: The start_date of this Education.
        :type: FuzzyDate
        """

        self._start_date = start_date

    @property
    def end_date(self):
        """
        Gets the end_date of this Education.

        :return: The end_date of this Education.
        :rtype: FuzzyDate
        """
        return self._end_date

    @end_date.setter
    def end_date(self, end_date):
        """
        Sets the end_date of this Education.

        :param end_date: The end_date of this Education.
        :type: FuzzyDate
        """

        self._end_date = end_date

    @property
    def organization(self):
        """
        Gets the organization of this Education.

        :return: The organization of this Education.
        :rtype: Organization
        """
        return self._organization

    @organization.setter
    def organization(self, organization):
        """
        Sets the organization of this Education.

        :param organization: The organization of this Education.
        :type: Organization
        """
        if organization is None:
            raise ValueError("Invalid value for `organization`, must not be `None`")

        self._organization = organization

    @property
    def visibility(self):
        """
        Gets the visibility of this Education.

        :return: The visibility of this Education.
        :rtype: str
        """
        return self._visibility

    @visibility.setter
    def visibility(self, visibility):
        """
        Sets the visibility of this Education.

        :param visibility: The visibility of this Education.
        :type: str
        """
        allowed_values = ["LIMITED", "REGISTERED_ONLY", "PUBLIC", "PRIVATE"]
        if visibility not in allowed_values:
            raise ValueError(
                "Invalid value for `visibility` ({0}), must be one of {1}"
                .format(visibility, allowed_values)
            )

        self._visibility = visibility

    def to_dict(self):
        """
        Returns the model properties as a dict
        """
        result = {}

        for attr, _ in iteritems(self.swagger_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value

        return result

    def to_str(self):
        """
        Returns the string representation of the model
        """
        return pformat(self.to_dict())

    def __repr__(self):
        """
        For `print` and `pprint`
        """
        return self.to_str()

    def __eq__(self, other):
        """
        Returns true if both objects are equal
        """
        if not isinstance(other, Education):
            return False

        return self.__dict__ == other.__dict__

    def __ne__(self, other):
        """
        Returns true if both objects are not equal
        """
        return not self == other
